@@ -1,9 +1,12 @@
 import React from 'react';
 import { cx } from 'emotion';
+import _ from 'lodash';
 import { SelectableValue } from '@grafana/data';
 import { SegmentSelect, useExpandableLabel, SegmentProps } from './';
 
 export interface SegmentSyncProps<T> extends SegmentProps<T> {
+  value?: T | SelectableValue<T>;
+  onChange: (item: SelectableValue<T>) => void;
   options: Array<SelectableValue<T>>;
 }
 
@@ -14,22 +17,35 @@ export function Segment<T>({
   Component,
   className,
   allowCustomValue,
+  placeholder,
 }: React.PropsWithChildren<SegmentSyncProps<T>>) {
   const [Label, width, expanded, setExpanded] = useExpandableLabel(false);
 
   if (!expanded) {
-    return <Label Component={Component || <a className={cx('gf-form-label', 'query-part', className)}>{value}</a>} />;
+    const label = _.isObject(value) ? value.label : value;
+    return (
+      <Label
+        Component={
+          Component || (
+            <a className={cx('gf-form-label', 'query-part', !value && placeholder && 'query-placeholder', className)}>
+              {label || placeholder}
+            </a>
+          )
+        }
+      />
+    );
   }
 
   return (
     <SegmentSelect
-      width={width}
+      value={value && !_.isObject(value) ? { value } : value}
       options={options}
+      width={width}
       onClickOutside={() => setExpanded(false)}
       allowCustomValue={allowCustomValue}
-      onChange={value => {
+      onChange={item => {
         setExpanded(false);
-        onChange(value);
+        onChange(item);
       }}
     />
   );
