@@ -1,13 +1,14 @@
-import { DataQuery, DataSourceJsonData } from '@grafana/ui';
+import { DataQuery, DataSourceJsonData } from '@grafana/data';
 
 export interface ElasticsearchOptions extends DataSourceJsonData {
   timeField: string;
   esVersion: number;
-  interval: string;
+  interval?: string;
   timeInterval: string;
   maxConcurrentShardRequests?: number;
   logMessageField?: string;
   logLevelField?: string;
+  dataLinks?: DataLinkConfig[];
 }
 
 export interface ElasticsearchAggregation {
@@ -24,3 +25,9 @@ export interface ElasticsearchQuery extends DataQuery {
   bucketAggs?: ElasticsearchAggregation[];
   metrics?: ElasticsearchAggregation[];
 }
+
+export type DataLinkConfig = {
+  field: string;
+  url: string;
+  datasourceUid?: string;
+};
